# dynamic-scope

Create dynamic scope in JavaScript with "with statement" trick.

It is a native ES2017 module, it provides complete dynamic scope implementation in non-strict mode and incomplete dynamic scope implementation in strict mode.

## Installation

```sh
yarn add dynamic-scope
```

## Usage

```javascript
import createDynamicScopeExpression from 'dynamic-scope'

function someFuncShouldBeDynamicScope(...args) {
  console.log([message, ...args].join(' ') + '!')
}

const dynamicFuncExpression = createDynamicScopeExpression(someFuncShouldBeDynamicScope)

// Non-strict mode
;(() => {
  const message = 'Hello'
  const func = eval(dynamicFuncExpression)()
  func('World') // Hello World!
})()

// Strict mode
;(() => {
  'use strict'
  const func = eval(dynamicFuncExpression)({ message: 'Hello' })
  func('World') // Hello World!
})()
```

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

-   [createDynamicScopeExpression](#createdynamicscopeexpression)

### createDynamicScopeExpression

Create dynamic scope in JavaScript with "with statement" trick.

**Parameters**

-   `fn` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** The function needs to convert

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** An expression needs eval() to active, eval(expression) will return (context) => function
