"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const babylon = require("babylon");
function parse(code) {
    return babylon.parseExpression(code, {
        plugins: ['asyncGenerators']
    });
}
function createNewFunctionExpression(code, isStrictMode) {
    return isStrictMode
        ? `new Function(${JSON.stringify(code)})`
        : `eval(${JSON.stringify(`(function(context) { ${code} })`)})`;
}
function isNativeFunction(code) {
    return /{\s+\[native code\]\s+}$/.test(code);
}
/**
 * Create dynamic scope in JavaScript with "with statement" trick.
 *
 * @param{function} fn - The function need to convert
 * @return{string} An expression need eval() to active
 */
function createDynamicScopeExpression(fn) {
    const sourceCode = Function.prototype.toString.call(fn);
    if (isNativeFunction(sourceCode)) {
        throw new TypeError('Native function cannot convert to dynamic scope.');
    }
    const callableCode = (() => {
        try {
            // function or arrow function
            parse(sourceCode);
            return `(${sourceCode})`;
        }
        catch (e) {
            // class method
            const ast = parse(`{${sourceCode}}`);
            const method = ast.properties[0];
            const key = method.key;
            let name;
            if ('value' in key) {
                name = key.value;
            }
            if ('name' in key) {
                name = key.name;
            }
            if ('quasis' in key) {
                name = key.quasis[0].value.cooked;
            }
            return `({${sourceCode}})[${JSON.stringify(name)}]`;
        }
    })();
    const dynamicScopeCode = `
    with(/* context */arguments[arguments.length - 2]) {
      return Reflect.apply(
        eval(${JSON.stringify(`(${callableCode})`)})
      , this
      , /* args */arguments[arguments.length - 1]
      )
    }
  `;
    const finalExpression = `
    ((context = {}) => function() {
      if ((() => {
        try {
          eval('00') // Octal literals are not allowed in strict mode
          return false
        } catch(e) {
          return true
        }
      })()) {
        // Strict mode
        // For unlock strict mode but also escape closure(incomplete dynamic scope)
        return ${createNewFunctionExpression(dynamicScopeCode, true)}.call(this
        , /* context */context
        , /* args */arguments
        )
      } else {
        // Non-strict mode
        // Keep closure so should blocked variables(complete dynamic scope)
        return ${createNewFunctionExpression(dynamicScopeCode, false)}.call(this
        , undefined
        , /* context */context
        , /* args */arguments
        )
      }
    })
  `;
    return finalExpression;
}
exports.createDynamicScopeExpression = createDynamicScopeExpression;
exports.default = createDynamicScopeExpression;
//# sourceMappingURL=dynamic-scope.js.map