import { createDynamicScopeExpression } from '../../../src/dynamic-scope'

test('Class normal name generator function: * named() {}', () => {
  const obj = {
    * named() {
      yield message
    }
  }

  const scope = createDynamicScopeExpression(obj.named)
  const func = eval(scope)({ message: 'Hello World!' })

  expect([...func()]).toEqual(['Hello World!'])
})

test("Class unnormal name generator function: * ['*']() {}", () => {
  const obj = {
    * ['*']() {
      yield message
    }
  }

  const scope = createDynamicScopeExpression(obj['*'])
  const func = eval(scope)({ message: 'Hello World!' })

  expect([...func()]).toEqual(['Hello World!'])
})

test('Class unnormal name generator function: * ["*"]() {}', () => {
  const obj = {
    * ["*"]() {
      yield message
    }
  }

  const scope = createDynamicScopeExpression(obj['*'])
  const func = eval(scope)({ message: 'Hello World!' })

  expect([...func()]).toEqual(['Hello World!'])
})

test('Class unnormal name generator function: * [`*`]() {}', () => {
  const obj = {
    * [`*`]() {
      yield message
    }
  }

  const scope = createDynamicScopeExpression(obj['*'])
  const func = eval(scope)({ message: 'Hello World!' })

  expect([...func()]).toEqual(['Hello World!'])
})

test('Class normal name async generator function: async * named() {}', async () => {
  const obj = {
    async * named() {
      for await (const i of [1, 2, 3]) {
        yield i
      }
    }
  }

  const scope = createDynamicScopeExpression(obj.named)
  const func = eval(scope)({ message: 'Hello World!' })

  const result = []
  for await (const i of func()) {
    result.push(i)
  }

  expect(result).toEqual([1, 2, 3])
})

test("Class unnormal name async generator function: async * ['*']() {", async () => {
  const obj = {
    async * ['*']() {
      for await (const i of [1, 2, 3]) {
        yield i
      }
    }
  }

  const scope = createDynamicScopeExpression(obj['*'])
  const func = eval(scope)({ message: 'Hello World!' })

  const result = []
  for await (const i of func()) {
    result.push(i)
  }

  expect(result).toEqual([1, 2, 3])
})

test('Class unnormal name async generator function: async * ["*"]() {', async () => {
  const obj = {
    async * ["*"]() {
      for await (const i of [1, 2, 3]) {
        yield i
      }
    }
  }

  const scope = createDynamicScopeExpression(obj['*'])
  const func = eval(scope)({ message: 'Hello World!' })

  const result = []
  for await (const i of func()) {
    result.push(i)
  }

  expect(result).toEqual([1, 2, 3])
})

test('Class unnormal name async generator function: async * [`*`]() {', async () => {
  const obj = {
    async * ["*"]() {
      for await (const i of [1, 2, 3]) {
        yield i
      }
    }
  }

  const scope = createDynamicScopeExpression(obj['*'])
  const func = eval(scope)({ message: 'Hello World!' })

  const result = []
  for await (const i of func()) {
    result.push(i)
  }

  expect(result).toEqual([1, 2, 3])
})
