import { createDynamicScopeExpression } from '../../src/dynamic-scope'

test('Generator function: function* named() {}', () => {
  function* named() {
    yield message
  }

  const scope = createDynamicScopeExpression(named)
  const func = eval(scope)({ message: 'Hello World!' })

  expect([...func()]).toEqual(['Hello World!'])
})

test('Anonymous generator function: function* () {}', () => {
  const anonymous = function*() {
    yield message
  }

  const scope = createDynamicScopeExpression(anonymous)
  const func = eval(scope)({ message: 'Hello World!' })

  expect([...func()]).toEqual(['Hello World!'])
})

test('Async generator function: async function* named() {}', async function() {
  async function* named() {
    for await (const i of [1, 2, 3]) {
      yield i
    }
  }

  const scope = createDynamicScopeExpression(named)
  const func = eval(scope)({ message: 'Hello World!' })

  const result = []
  for await (const i of func()) {
    result.push(i)
  }

  expect(result).toEqual([1, 2, 3])
})

test('Anonymous async generator function: async function* () {}', async function() {
  const anonymous = async function*() {
    for await (const i of [1, 2, 3]) {
      yield i
    }
  }

  const scope = createDynamicScopeExpression(anonymous)
  const func = eval(scope)({ message: 'Hello World!' })

  const result = []
  for await (const i of func()) {
    result.push(i)
  }

  expect(result).toEqual([1, 2, 3])
})
