import { createDynamicScopeExpression } from '../src/dynamic-scope'

describe('Usage example', () => {
  test('non-strict mode', () => {
    function someFuncShouldBeDynamicScope(...args) {
      return [message, ...args].join(' ') + '!'
    }
  
    const dynamicFuncExpression = createDynamicScopeExpression(someFuncShouldBeDynamicScope)

    ;(() => {
      const message = 'Hello'
      const func = eval(dynamicFuncExpression)()
      expect(func('World')).toEqual('Hello World!') // Hello World!
    })()
  })

  test('non-strict mode with context', () => {
    const message = 'Goodbye'
    
    function someFuncShouldBeDynamicScope(...args) {
      return [message, ...args].join(' ') + '!'
    }
  
    const dynamicFuncExpression = createDynamicScopeExpression(someFuncShouldBeDynamicScope)

    ;(() => {
      const message = 'Hello'
      const func = eval(dynamicFuncExpression)({ message: 'Hello' })
      expect(func('World')).toEqual('Hello World!') // Hello World!
    })()
  })

  test('strict mode', () => {
    function someFuncShouldBeDynamicScope(...args) {
      return [message, ...args].join(' ') + '!'
    }
  
    const dynamicFuncExpression = createDynamicScopeExpression(someFuncShouldBeDynamicScope)

    ;(() => {
      'use strict'
      const func = eval(dynamicFuncExpression)({ message: 'Hello' })
      expect(func('World')).toEqual('Hello World!') // Hello World!
    })()
  })
})
